@@ -2,7 +2,7 @@
 # pylint: disable=R0902,R0912,R0913
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2016
+# Copyright (C) 2015-2017
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -18,13 +18,15 @@
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains an object that represents a Telegram Message."""
-
 import sys
 from datetime import datetime
 from time import mktime
 
 from telegram import (Audio, Contact, Document, Chat, Location, PhotoSize, Sticker, TelegramObject,
-                      User, Video, Voice, Venue, MessageEntity, Game)
+                      User, Video, Voice, Venue, MessageEntity, Game, Invoice, SuccessfulPayment)
+from telegram.utils.deprecate import warn_deprecate_obj
+from telegram.utils.helpers import escape_html, escape_markdown
+from telegram.videonote import VideoNote
 
 
 class Message(TelegramObject):
@@ -34,36 +36,69 @@ class Message(TelegramObject):
         * In Python `from` is a reserved word, use `from_user` instead.
 
     Attributes:
-        message_id (int):
-        from_user (:class:`telegram.User`):
-        date (:class:`datetime.datetime`):
-        forward_from (:class:`telegram.User`):
-        forward_from_chat (:class:`telegram.Chat`):
-        forward_from_message_id (int):
-        forward_date (:class:`datetime.datetime`):
-        reply_to_message (:class:`telegram.Message`):
-        edit_date (:class:`datetime.datetime`):
-        text (str):
-        audio (:class:`telegram.Audio`):
-        document (:class:`telegram.Document`):
-        game (:class:`telegram.Game`):
-        photo (List[:class:`telegram.PhotoSize`]):
-        sticker (:class:`telegram.Sticker`):
-        video (:class:`telegram.Video`):
-        voice (:class:`telegram.Voice`):
-        caption (str):
-        contact (:class:`telegram.Contact`):
-        location (:class:`telegram.Location`):
-        new_chat_member (:class:`telegram.User`):
-        left_chat_member (:class:`telegram.User`):
-        new_chat_title (str):
-        new_chat_photo (List[:class:`telegram.PhotoSize`]):
-        delete_chat_photo (bool):
-        group_chat_created (bool):
-        supergroup_chat_created (bool):
-        migrate_to_chat_id (int):
-        migrate_from_chat_id (int):
-        channel_chat_created (bool):
+        message_id (int): Unique message identifier inside this chat
+        from_user (:class:`telegram.User`): Sender, can be empty for messages sent to channels
+        date (:class:`datetime.datetime`): Date the message was sent in Unix time
+        chat (:class:`telegram.Chat`): Conversation the message belongs to
+        forward_from (:class:`telegram.User`): For forwarded messages, sender of the original
+            message
+        forward_from_chat (:class:`telegram.Chat`): For messages forwarded from a channel,
+            information about the original channel
+        forward_from_message_id (int): For forwarded channel posts, identifier of the original
+            message in the channel
+        forward_date (:class:`datetime.datetime`): For forwarded messages, date the original
+            message was sent in Unix time
+        reply_to_message (:class:`telegram.Message`): For replies, the original message. Note
+            that the Message object in this field will not contain further reply_to_message
+            fields even if it itself is a reply.
+        edit_date (:class:`datetime.datetime`): Date the message was last edited in Unix time
+        text (str): For text messages, the actual UTF-8 text of the message, 0-4096 characters.
+        entities (List[:class:`telegram.MessageEntity`]): For text messages, special entities
+            like usernames, URLs, bot commands, etc. that appear in the text. See
+            parse_entity and parse_entities methods for how to use properly
+        video_note (:class:`telegram.VideoNote`): Message is a video note, information about the
+            video message
+        audio (:class:`telegram.Audio`): Message is an audio file, information about the file
+        document (:class:`telegram.Document`): Message is a general file, information about the
+            file
+        game (:class:`telegram.Game`):Message is a game, information about the game
+        photo (List[:class:`telegram.PhotoSize`]): Message is a photo, available sizes of the photo
+        sticker (:class:`telegram.Sticker`): Message is a sticker, information about the sticker
+        video (:class:`telegram.Video`): Message is a video, information about the video
+        voice (:class:`telegram.Voice`): Message is a voice message, information about the file
+        caption (str): Caption for the document, photo or video, 0-200 characters
+        contact (:class:`telegram.Contact`): Message is a shared contact, information about the
+            contact
+        location (:class:`telegram.Location`): Message is a shared location, information about the
+            location
+        new_chat_member (:class:`telegram.User`): A new member was added to the group,
+            information about them (this member may be the bot itself)
+        left_chat_member (:class:`telegram.User`): A member was removed from the group,
+            information about them (this member may be the bot itself)
+        new_chat_title (str): A chat title was changed to this value
+        new_chat_photo (List[:class:`telegram.PhotoSize`]): A chat photo was change to this value
+        delete_chat_photo (bool): Service message: the chat photo was deleted
+        group_chat_created (bool): Service message: the group has been created
+        supergroup_chat_created (bool): Service message: the supergroup has been created. This
+            field can't be received in a message coming through updates, because bot can't be a
+            member of a supergroup when it is created. It can only be found in reply_to_message
+            if someone replies to a very first message in a directly created supergroup.
+        migrate_to_chat_id (int): The group has been migrated to a supergroup with the specified
+            identifier.
+        migrate_from_chat_id (int): The supergroup has been migrated from a group with the
+            specified identifier.
+        channel_chat_created (bool): Service message: the channel has been created. This field
+            can't be received in a message coming through updates, because bot can't be a member
+            of a channel when it is created. It can only be found in reply_to_message if someone
+            replies to a very first message in a channel.
+        pinned_message (:class:`telegram.message`): Specified message was pinned. Note that the
+            Message object in this field will not contain further reply_to_message fields even if
+            it is itself a reply.
+        invoice (:class:`telegram.Invoice`): Message is an invoice for a payment, information
+            about the invoice.
+        successful_payment (:class:`telegram.SuccessfulPayment`): Message is a service message
+            about a successful payment, information about the payment.
+        bot (Optional[telegram.Bot]): The Bot to use for instance methods
 
     Deprecated: 4.0
         new_chat_participant (:class:`telegram.User`): Use `new_chat_member`
@@ -71,40 +106,6 @@ class Message(TelegramObject):
 
         left_chat_participant  (:class:`telegram.User`): Use `left_chat_member`
         instead.
-
-    Args:
-        message_id (int):
-        from_user (:class:`telegram.User`):
-        date (:class:`datetime.datetime`):
-        chat (:class:`telegram.Chat`):
-        forward_from (Optional[:class:`telegram.User`]):
-        forward_from_chat (Optional[:class:`telegram.Chat`]):
-        forward_from_message_id (Optional[int]):
-        forward_date (Optional[:class:`datetime.datetime`]):
-        reply_to_message (Optional[:class:`telegram.Message`]):
-        edit_date (Optional[:class:`datetime.datetime`]):
-        text (Optional[str]):
-        audio (Optional[:class:`telegram.Audio`]):
-        document (Optional[:class:`telegram.Document`]):
-        game (Optional[:class:`telegram.Game`]):
-        photo (Optional[List[:class:`telegram.PhotoSize`]]):
-        sticker (Optional[:class:`telegram.Sticker`]):
-        video (Optional[:class:`telegram.Video`]):
-        voice (Optional[:class:`telegram.Voice`]):
-        caption (Optional[str]):
-        contact (Optional[:class:`telegram.Contact`]):
-        location (Optional[:class:`telegram.Location`]):
-        new_chat_member (Optional[:class:`telegram.User`]):
-        left_chat_member (Optional[:class:`telegram.User`]):
-        new_chat_title (Optional[str]):
-        new_chat_photo (Optional[List[:class:`telegram.PhotoSize`]):
-        delete_chat_photo (Optional[bool]):
-        group_chat_created (Optional[bool]):
-        supergroup_chat_created (Optional[bool]):
-        migrate_to_chat_id (Optional[int]):
-        migrate_from_chat_id (Optional[int]):
-        channel_chat_created (Optional[bool]):
-        bot (Optional[Bot]): The Bot to use for instance methods
     """
 
     def __init__(self,
@@ -117,7 +118,7 @@ class Message(TelegramObject):
                  forward_date=None,
                  reply_to_message=None,
                  edit_date=None,
-                 text='',
+                 text=None,
                  entities=None,
                  audio=None,
                  document=None,
@@ -125,23 +126,27 @@ class Message(TelegramObject):
                  sticker=None,
                  video=None,
                  voice=None,
-                 caption='',
+                 caption=None,
                  contact=None,
                  location=None,
                  venue=None,
                  new_chat_member=None,
+                 new_chat_members=None,
                  left_chat_member=None,
-                 new_chat_title='',
+                 new_chat_title=None,
                  new_chat_photo=None,
                  delete_chat_photo=False,
                  group_chat_created=False,
                  supergroup_chat_created=False,
-                 migrate_to_chat_id=0,
-                 migrate_from_chat_id=0,
+                 migrate_to_chat_id=None,
+                 migrate_from_chat_id=None,
                  channel_chat_created=False,
                  pinned_message=None,
                  forward_from_message_id=None,
+                 invoice=None,
+                 successful_payment=None,
                  bot=None,
+                 video_note=None,
                  **kwargs):
         # Required
         self.message_id = int(message_id)
@@ -163,25 +168,31 @@ class Message(TelegramObject):
         self.sticker = sticker
         self.video = video
         self.voice = voice
+        self.video_note = video_note
         self.caption = caption
         self.contact = contact
         self.location = location
         self.venue = venue
-        self.new_chat_member = new_chat_member
+        self._new_chat_member = new_chat_member
+        self.new_chat_members = new_chat_members
         self.left_chat_member = left_chat_member
         self.new_chat_title = new_chat_title
         self.new_chat_photo = new_chat_photo
         self.delete_chat_photo = bool(delete_chat_photo)
         self.group_chat_created = bool(group_chat_created)
         self.supergroup_chat_created = bool(supergroup_chat_created)
-        self.migrate_to_chat_id = int(migrate_to_chat_id)
-        self.migrate_from_chat_id = int(migrate_from_chat_id)
+        self.migrate_to_chat_id = migrate_to_chat_id
+        self.migrate_from_chat_id = migrate_from_chat_id
         self.channel_chat_created = bool(channel_chat_created)
         self.pinned_message = pinned_message
         self.forward_from_message_id = forward_from_message_id
+        self.invoice = invoice
+        self.successful_payment = successful_payment
 
         self.bot = bot
 
+        self._id_attrs = (self.message_id,)
+
     @property
     def chat_id(self):
         """int: Short for :attr:`Message.chat.id`"""
@@ -200,8 +211,10 @@ class Message(TelegramObject):
         if not data:
             return None
 
+        data = super(Message, Message).de_json(data, bot)
+
         data['from_user'] = User.de_json(data.get('from'), bot)
-        data['date'] = datetime.fromtimestamp(data['date'])
+        data['date'] = Message._fromtimestamp(data['date'])
         data['chat'] = Chat.de_json(data.get('chat'), bot)
         data['entities'] = MessageEntity.de_list(data.get('entities'), bot)
         data['forward_from'] = User.de_json(data.get('forward_from'), bot)
@@ -216,13 +229,17 @@ class Message(TelegramObject):
         data['sticker'] = Sticker.de_json(data.get('sticker'), bot)
         data['video'] = Video.de_json(data.get('video'), bot)
         data['voice'] = Voice.de_json(data.get('voice'), bot)
+        data['video_note'] = VideoNote.de_json(data.get('video_note'), bot)
         data['contact'] = Contact.de_json(data.get('contact'), bot)
         data['location'] = Location.de_json(data.get('location'), bot)
         data['venue'] = Venue.de_json(data.get('venue'), bot)
         data['new_chat_member'] = User.de_json(data.get('new_chat_member'), bot)
+        data['new_chat_members'] = User.de_list(data.get('new_chat_members'), bot)
         data['left_chat_member'] = User.de_json(data.get('left_chat_member'), bot)
         data['new_chat_photo'] = PhotoSize.de_list(data.get('new_chat_photo'), bot)
         data['pinned_message'] = Message.de_json(data.get('pinned_message'), bot)
+        data['invoice'] = Invoice.de_json(data.get('invoice'), bot)
+        data['successful_payment'] = SuccessfulPayment.de_json(data.get('successful_payment'), bot)
 
         return Message(bot=bot, **data)
 
@@ -253,6 +270,9 @@ class Message(TelegramObject):
             data['entities'] = [e.to_dict() for e in self.entities]
         if self.new_chat_photo:
             data['new_chat_photo'] = [p.to_dict() for p in self.new_chat_photo]
+        data['new_chat_member'] = data.pop('_new_chat_member', None)
+        if self.new_chat_members:
+            data['new_chat_members'] = [u.to_dict() for u in self.new_chat_members]
 
         return data
 
@@ -308,7 +328,7 @@ class Message(TelegramObject):
 
     def reply_text(self, *args, **kwargs):
         """
-        Shortcut for ``bot.sendMessage(update.message.chat_id, *args, **kwargs)``
+        Shortcut for ``bot.send_message(update.message.chat_id, *args, **kwargs)``
 
         Keyword Args:
             quote (Optional[bool]): If set to ``True``, the message is sent as an actual reply to
@@ -317,11 +337,11 @@ class Message(TelegramObject):
         """
 
         self._quote(kwargs)
-        return self.bot.sendMessage(self.chat_id, *args, **kwargs)
+        return self.bot.send_message(self.chat_id, *args, **kwargs)
 
     def reply_photo(self, *args, **kwargs):
         """
-        Shortcut for ``bot.sendPhoto(update.message.chat_id, *args, **kwargs)``
+        Shortcut for ``bot.send_photo(update.message.chat_id, *args, **kwargs)``
 
         Keyword Args:
             quote (Optional[bool]): If set to ``True``, the photo is sent as an actual reply to
@@ -334,11 +354,11 @@ class Message(TelegramObject):
         """
 
         self._quote(kwargs)
-        return self.bot.sendPhoto(self.chat_id, *args, **kwargs)
+        return self.bot.send_photo(self.chat_id, *args, **kwargs)
 
     def reply_audio(self, *args, **kwargs):
         """
-        Shortcut for ``bot.sendAudio(update.message.chat_id, *args, **kwargs)``
+        Shortcut for ``bot.send_audio(update.message.chat_id, *args, **kwargs)``
 
         Keyword Args:
             quote (Optional[bool]): If set to ``True``, the audio is sent as an actual reply to
@@ -351,11 +371,11 @@ class Message(TelegramObject):
         """
 
         self._quote(kwargs)
-        return self.bot.sendAudio(self.chat_id, *args, **kwargs)
+        return self.bot.send_audio(self.chat_id, *args, **kwargs)
 
     def reply_document(self, *args, **kwargs):
         """
-        Shortcut for ``bot.sendDocument(update.message.chat_id, *args, **kwargs)``
+        Shortcut for ``bot.send_document(update.message.chat_id, *args, **kwargs)``
 
         Keyword Args:
             quote (Optional[bool]): If set to ``True``, the document is sent as an actual reply to
@@ -368,11 +388,11 @@ class Message(TelegramObject):
         """
 
         self._quote(kwargs)
-        return self.bot.sendDocument(self.chat_id, *args, **kwargs)
+        return self.bot.send_document(self.chat_id, *args, **kwargs)
 
     def reply_sticker(self, *args, **kwargs):
         """
-        Shortcut for ``bot.sendSticker(update.message.chat_id, *args, **kwargs)``
+        Shortcut for ``bot.send_sticker(update.message.chat_id, *args, **kwargs)``
 
         Keyword Args:
             quote (Optional[bool]): If set to ``True``, the sticker is sent as an actual reply to
@@ -385,11 +405,11 @@ class Message(TelegramObject):
         """
 
         self._quote(kwargs)
-        return self.bot.sendSticker(self.chat_id, *args, **kwargs)
+        return self.bot.send_sticker(self.chat_id, *args, **kwargs)
 
     def reply_video(self, *args, **kwargs):
         """
-        Shortcut for ``bot.sendVideo(update.message.chat_id, *args, **kwargs)``
+        Shortcut for ``bot.send_video(update.message.chat_id, *args, **kwargs)``
 
         Keyword Args:
             quote (Optional[bool]): If set to ``True``, the video is sent as an actual reply to
@@ -402,11 +422,28 @@ class Message(TelegramObject):
         """
 
         self._quote(kwargs)
-        return self.bot.sendVideo(self.chat_id, *args, **kwargs)
+        return self.bot.send_video(self.chat_id, *args, **kwargs)
+
+    def reply_video_note(self, *args, **kwargs):
+        """
+        Shortcut for ``bot.send_video_note(update.message.chat_id, *args, **kwargs)``
+
+        Keyword Args:
+            quote (Optional[bool]): If set to ``True``, the video is sent as an actual reply to
+                this message. If ``reply_to_message_id`` is passed in ``kwargs``, this parameter
+                will be ignored. Default: ``True`` in group chats and ``False`` in private chats.
+
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+
+        self._quote(kwargs)
+        return self.bot.send_video_note(self.chat_id, *args, **kwargs)
 
     def reply_voice(self, *args, **kwargs):
         """
-        Shortcut for ``bot.sendVoice(update.message.chat_id, *args, **kwargs)``
+        Shortcut for ``bot.send_voice(update.message.chat_id, *args, **kwargs)``
 
         Keyword Args:
             quote (Optional[bool]): If set to ``True``, the voice is sent as an actual reply to
@@ -419,11 +456,11 @@ class Message(TelegramObject):
         """
 
         self._quote(kwargs)
-        return self.bot.sendVoice(self.chat_id, *args, **kwargs)
+        return self.bot.send_voice(self.chat_id, *args, **kwargs)
 
     def reply_location(self, *args, **kwargs):
         """
-        Shortcut for ``bot.sendLocation(update.message.chat_id, *args, **kwargs)``
+        Shortcut for ``bot.send_location(update.message.chat_id, *args, **kwargs)``
 
         Keyword Args:
             quote (Optional[bool]): If set to ``True``, the location is sent as an actual reply to
@@ -436,11 +473,11 @@ class Message(TelegramObject):
         """
 
         self._quote(kwargs)
-        return self.bot.sendLocation(self.chat_id, *args, **kwargs)
+        return self.bot.send_location(self.chat_id, *args, **kwargs)
 
     def reply_venue(self, *args, **kwargs):
         """
-        Shortcut for ``bot.sendVenue(update.message.chat_id, *args, **kwargs)``
+        Shortcut for ``bot.send_venue(update.message.chat_id, *args, **kwargs)``
 
         Keyword Args:
             quote (Optional[bool]): If set to ``True``, the venue is sent as an actual reply to
@@ -453,11 +490,11 @@ class Message(TelegramObject):
         """
 
         self._quote(kwargs)
-        return self.bot.sendVenue(self.chat_id, *args, **kwargs)
+        return self.bot.send_venue(self.chat_id, *args, **kwargs)
 
     def reply_contact(self, *args, **kwargs):
         """
-        Shortcut for ``bot.sendContact(update.message.chat_id, *args, **kwargs)``
+        Shortcut for ``bot.send_contact(update.message.chat_id, *args, **kwargs)``
 
         Keyword Args:
             quote (Optional[bool]): If set to ``True``, the contact is sent as an actual reply to
@@ -470,12 +507,12 @@ class Message(TelegramObject):
         """
 
         self._quote(kwargs)
-        return self.bot.sendContact(self.chat_id, *args, **kwargs)
+        return self.bot.send_contact(self.chat_id, *args, **kwargs)
 
     def forward(self, chat_id, disable_notification=False):
         """Shortcut for
 
-            >>> bot.forwardMessage(chat_id=chat_id,
+            >>> bot.forward_message(chat_id=chat_id,
             ...                    from_chat_id=update.message.chat_id,
             ...                    disable_notification=disable_notification,
             ...                    message_id=update.message.message_id)
@@ -484,7 +521,7 @@ class Message(TelegramObject):
             :class:`telegram.Message`: On success, instance representing the message forwarded.
 
         """
-        return self.bot.forwardMessage(
+        return self.bot.forward_message(
             chat_id=chat_id,
             from_chat_id=self.chat_id,
             disable_notification=disable_notification,
@@ -494,7 +531,7 @@ class Message(TelegramObject):
         """
         Shortcut for
 
-            >>> bot.editMessageText(chat_id=message.chat_id,
+            >>> bot.edit_message_text(chat_id=message.chat_id,
             ...                     message_id=message.message_id,
             ...                     *args, **kwargs)
 
@@ -511,7 +548,7 @@ class Message(TelegramObject):
         """
         Shortcut for
 
-            >>> bot.editMessageCaption(chat_id=message.chat_id,
+            >>> bot.edit_message_caption(chat_id=message.chat_id,
             ...                        message_id=message.message_id,
             ...                        *args, **kwargs)
 
@@ -527,7 +564,7 @@ class Message(TelegramObject):
         """
         Shortcut for
 
-            >>> bot.editReplyMarkup(chat_id=message.chat_id,
+            >>> bot.edit_message_reply_markup(chat_id=message.chat_id,
             ...                     message_id=message.message_id,
             ...                     *args, **kwargs)
 
@@ -539,6 +576,21 @@ class Message(TelegramObject):
         return self.bot.edit_message_reply_markup(
             chat_id=self.chat_id, message_id=self.message_id, *args, **kwargs)
 
+    def delete(self, *args, **kwargs):
+        """
+        Shortcut for
+
+            >>> bot.delete_message(chat_id=message.chat_id,
+            ...                   message_id=message.message_id,
+            ...                   *args, **kwargs)
+
+        Returns:
+            bool: On success, `True` is returned.
+
+        """
+        return self.bot.delete_message(
+            chat_id=self.chat_id, message_id=self.message_id, *args, **kwargs)
+
     def parse_entity(self, entity):
         """
         Returns the text from a given :class:`telegram.MessageEntity`.
@@ -584,6 +636,7 @@ class Message(TelegramObject):
         Returns:
             dict[:class:`telegram.MessageEntity`, ``str``]: A dictionary of entities mapped to the
                 text that belongs to them, calculated based on UTF-16 codepoints.
+
         """
         if types is None:
             types = MessageEntity.ALL_TYPES
@@ -592,3 +645,108 @@ class Message(TelegramObject):
             entity: self.parse_entity(entity)
             for entity in self.entities if entity.type in types
         }
+
+    @property
+    def text_html(self):
+        """
+        Creates an html-formatted string from the markup entities found in the message
+        (uses ``parse_entities``).
+
+        Use this if you want to retrieve the original string sent by the bot, as opposed to the
+        plain text with corresponding markup entities.
+
+        Returns:
+            str
+
+        """
+        entities = self.parse_entities()
+        message_text = self.text
+        if not sys.maxunicode == 0xffff:
+            message_text = message_text.encode('utf-16-le')
+
+        markdown_text = ''
+        last_offset = 0
+
+        for entity, text in sorted(entities.items(), key=(lambda item: item[0].offset)):
+            text = escape_html(text)
+
+            if entity.type == MessageEntity.TEXT_LINK:
+                insert = '<a href="{}">{}</a>'.format(entity.url, text)
+            elif entity.type == MessageEntity.BOLD:
+                insert = '<b>' + text + '</b>'
+            elif entity.type == MessageEntity.ITALIC:
+                insert = '<i>' + text + '</i>'
+            elif entity.type == MessageEntity.CODE:
+                insert = '<code>' + text + '</code>'
+            elif entity.type == MessageEntity.PRE:
+                insert = '<pre>' + text + '</pre>'
+            else:
+                insert = text
+
+            if sys.maxunicode == 0xffff:
+                markdown_text += escape_html(message_text[last_offset:entity.offset]) + insert
+            else:
+                markdown_text += escape_html(message_text[last_offset * 2:entity.offset * 2]
+                                             .decode('utf-16-le')) + insert
+
+            last_offset = entity.offset + entity.length
+
+        if sys.maxunicode == 0xffff:
+            markdown_text += escape_html(message_text[last_offset:])
+        else:
+            markdown_text += escape_html(message_text[last_offset * 2:].decode('utf-16-le'))
+        return markdown_text
+
+    @property
+    def text_markdown(self):
+        """
+        Creates a markdown-formatted string from the markup entities found in the message
+        (uses ``parse_entities``).
+
+        Use this if you want to retrieve the original string sent by the bot, as opposed to the
+        plain text with corresponding markup entities.
+
+        Returns:
+            str
+        """
+        entities = self.parse_entities()
+        message_text = self.text
+        if not sys.maxunicode == 0xffff:
+            message_text = message_text.encode('utf-16-le')
+
+        markdown_text = ''
+        last_offset = 0
+
+        for entity, text in sorted(entities.items(), key=(lambda item: item[0].offset)):
+            text = escape_markdown(text)
+
+            if entity.type == MessageEntity.TEXT_LINK:
+                insert = '[{}]({})'.format(text, entity.url)
+            elif entity.type == MessageEntity.BOLD:
+                insert = '*' + text + '*'
+            elif entity.type == MessageEntity.ITALIC:
+                insert = '_' + text + '_'
+            elif entity.type == MessageEntity.CODE:
+                insert = '`' + text + '`'
+            elif entity.type == MessageEntity.PRE:
+                insert = '```' + text + '```'
+            else:
+                insert = text
+            if sys.maxunicode == 0xffff:
+                markdown_text += escape_markdown(message_text[last_offset:entity.offset]) + insert
+            else:
+                markdown_text += escape_markdown(message_text[last_offset * 2:entity.offset * 2]
+                                                 .decode('utf-16-le')) + insert
+
+            last_offset = entity.offset + entity.length
+
+        if sys.maxunicode == 0xffff:
+            markdown_text += escape_markdown(message_text[last_offset:])
+        else:
+            markdown_text += escape_markdown(message_text[last_offset * 2:].decode('utf-16-le'))
+        return markdown_text
+
+    @property
+    def new_chat_member(self):
+        warn_deprecate_obj('new_chat_member', 'new_chat_members')
+        return self._new_chat_member
