@@ -43,7 +43,7 @@ class User(TelegramObject):
         last_name (Optional[str]): User's or bot's last name
         username (Optional[str]): User's or bot's username
         language_code (Optional[str]): IETF language tag of the user's language
-        bot (Optional[Bot]): The Bot to use for instance methods
+        bot (Optional[telegram.Bot]): The Bot to use for instance methods
     """
 
     def __init__(self,
@@ -96,6 +96,25 @@ class User(TelegramObject):
 
     def get_profile_photos(self, *args, **kwargs):
         """
-        Shortcut for ``bot.getUserProfilePhotos(update.message.from_user.id, *args, **kwargs)``
+        Shortcut for ``bot.get_user_profile_photos(update.message.from_user.id, *args, **kwargs)``
         """
-        return self.bot.getUserProfilePhotos(self.id, *args, **kwargs)
+        return self.bot.get_user_profile_photos(self.id, *args, **kwargs)
+
+    @staticmethod
+    def de_list(data, bot):
+        """
+        Args:
+            data (list):
+            bot (telegram.Bot):
+
+        Returns:
+            List<telegram.User>:
+        """
+        if not data:
+            return []
+
+        users = list()
+        for user in data:
+            users.append(User.de_json(user, bot))
+
+        return users
